/**
 * vue-promised v0.2.3
 * (c) 2018 Eduardo San Martin Morote <posva13@gmail.com>
 * @license MIT
 */

function assert (condition, message) {
  if (!condition) {
    throw new Error(("[vue-promised] " + message))
  }
}

var index = {
  name: 'Promised',
  props: {
    promise: {
      // allow polyfied Promise
      validator: function (p) { return p && typeof p.then === 'function' && typeof p.catch === 'function'; },
    },
    promises: Array,
    pendingDelay: {
      type: [Number, String],
      default: 200,
    },
  },

  data: function () { return ({
    resolved: false,
    data: null,
    error: null,

    isDelayElapsed: false,
  }); },

  render: function render (h) {
    if (this.error instanceof Error || (this.error && this.error.length)) {
      assert(
        this.$scopedSlots && this.$scopedSlots.catch,
        'Provide exactly one scoped slot named "catch" for the rejected promise'
      );
      return this.$scopedSlots.catch(this.error)
    } else if (this.resolved) {
      var slot = this.$scopedSlots.default || this.$scopedSlots.then;
      assert(
        this.$scopedSlots && slot,
        'Provide exactly one default/then scoped slot for the resolved promise'
      );
      return slot.call(this, this.data)
    } else if (this.isDelayElapsed) {
      assert(
        (this.$slots.default && this.$slots.default.length === 1) ||
          (this.$slots.pending && this.$slots.pending.length === 1),
        'Provide exactly one default/pending slot with no `slot-scope` for the pending promise'
      );
      return this.$slots.default ? this.$slots.default[0] : this.$slots.pending[0]
    }

    // do not render anything
    return h()
  },

  watch: {
    promise: {
      handler: function handler (promise) {
        var this$1 = this;

        if (!promise) { return }
        this.resolved = false;
        this.error = null;
        this.setupDelay();
        promise
          .then(function (data) {
            if (this$1.promise === promise) {
              this$1.resolved = true;
              this$1.data = data;
            }
          })
          .catch(function (err) {
            if (this$1.promise === promise) { this$1.error = err; }
          });
      },
      immediate: true,
    },

    promises: {
      handler: function handler (promises, oldPromises) {
        var this$1 = this;

        if (!promises) { return }
        if (promises !== oldPromises) {
          // reset the map if there's a new array
          this.ongoingPromises = new Map();
          this.resolved = false;
          this.error = [];
          this.data = [];
          this.setupDelay();
        }
        // do not listen for already set up promises
        promises.filter(function (p) { return !this$1.ongoingPromises.has(p); }).forEach(function (p) {
          this$1.ongoingPromises.set(p, true);
          p.then(function (data) {
            if (this$1.promises === promises) {
              this$1.resolved = true;
              this$1.data.push(data);
            }
          }).catch(function (err) {
            if (this$1.promises === promises) { this$1.error.push(err); }
          });
        });
      },
      immediate: true,
    },
  },

  methods: {
    setupDelay: function setupDelay () {
      var this$1 = this;

      if (this.pendingDelay > 0) {
        this.isDelayElapsed = false;
        if (this.timerId) { clearTimeout(this.timerId); }
        this.timerId = setTimeout(function () { return (this$1.isDelayElapsed = true); }, this.pendingDelay);
      } else {
        this.isDelayElapsed = true;
      }
    },
  },
};

export default index;
